/* ============================================================
 *
 * jquery.dpm.js v0.6.0
 * https://github.com/calvellido/DPMbox
 * Copyright (c) 2014 Juan Valencia Calvellido (juanvalenciacalvellido@gmail.com)
 *
 * ============================================================
 *
 * This library is based on jquery.dav, licensed under the Apache License, Version 2.0.
 * https://github.com/sandro-pasquali/jquery.dav
 * Copyright (c) 2011 Sandro Pasquali (spasquali@gmail.com)
 *
 * ============================================================
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * ============================================================ */


(function ($) {

    //Some directives for JSHint (JS code analyzer)
    'use strict';
    /* jshint browser: true, devel: true, jquery: true, eqeqeq: true, maxerr: 1000, quotmark: single, multistr: true*/

    /* The DAV methods you want to provide support for.
     *
     * @see #methodSupported
     */
    var supportedMethods = {
        GET:                1,
        POST:               1,
        HEAD:               2,
        PUT:                2,
        DELETE:             2,
        PROPFIND:           3,
        PROPPATCH:          3,
        MKCOL:              3,
        LOCK:               3,
        UNLOCK:             3,
        COPY:               3,
        MOVE:               3,
        REPORT:             3,
        SEARCH:             3,
        CHECKIN:            3,
        CHECKOUT:           3,
        UNCHECKOUT:         3,
        'VERSION-CONTROL':  3,
        TRACE:              3,
        BIND:               3,
        UNBIND:             3,
        REBIND:             3,
        MKREDIRECTREF:      3,
        OPTIONS:            3
    };

    /* Locking default settings.
     *
     * @see     #lock
     */
    var defaultLockScope = 'exclusive';
    var defaultLockType = 'write';
    var defaultLockTimeout = 86400;
    var defaultLockDepth = 'infinity';

    /* User home directory.
     *
     * @see     #lock
     */
    var userHomeDir = '/home/';

    /* XML header, used variously when making calls.
     */
    var XmlHeader = '<?xml version="1.0" encoding="utf-8" ?>';

    /* This is unfortunate, but I don't want to use the jquery browser detection
     * functions, as they are deprecated, and I don't know of a non-sniffing
     * way to determine whether or not this or that browser supports a given
     * method.  One dismissed consideration was to just support GET and POST,
     * and have *all* other methods use the POST?webdav-method construct, obviating
     * the need to check browser type.  If you would like to go that route, it
     * isn't hard to refactor into the POST?webdav-method construct.  NOTE: that this
     * POST behaviour is *exclusive* to the LimeBits implementation, via Hammock.
     */
    var uA = window.navigator.userAgent.toLowerCase();
    var isSafari = /webkit/.test(uA);
    var isOpera = /opera/.test(uA);
    var isIE = /msie/.test(uA) && !/opera/.test(uA);
    var isMoz = /mozilla/.test(uA) && !/(compatible|webkit)/.test(uA);
    var isGecko = /Gecko/.test(uA) && (/KHTML/.test(uA) === false);

    /* Utility functions
     */

    var isObject = function (ob) {
        return !!ob && Object.prototype.toString.call(ob) === '[object Object]';
    };

    var isArray = function (ob) {
        return !!ob && Object.prototype.toString.apply(ob) === '[object Array]';
    };

    var isString = function (ob) {
        return !!ob && Object.prototype.toString.apply(ob) === '[object String]';
    };

    /* Prior to making a call, check if the requested method is supported
     * by the current browser.
     *
     * @see #supportedMethods
     */
    var methodSupported = function (meth) {
        var sm = supportedMethods[meth];
        if (sm) {
            if (sm < 2) return true;
            if (isOpera && (sm < 3)) return true;
            if (isIE && (sm < 4)) return true;
            if (isMoz || isGecko || isSafari) return true;
        }
        return false;
    };

    /* jQuery#ajax option #beforeSend may be set to a function reference,
     * and this function will fire prior to the request being sent.  It is
     * passed the Xhr object, so this is the point you will want to do
     * things like set headers, for example.
     *
     * @see #propFind
     * @url http://docs.jquery.com/Ajax/jQuery.ajax#options
     */
    var extendBeforeSend = function (cob, f) {
        var bs = cob.beforeSend || function() {};

        cob.beforeSend = function (xhr) {
            bs(xhr);
            f(xhr);
        };
    };

    /***************************************
     * BEGIN #Dav extension
     ***************************************/

    $.fn.extend($, {

        /* The 'setup' method, which establishes the resource to be
         * operated on.
         *
         * @param     {Mixed}     res    A resource Url string, or a DOM Collection.
         * @returns                      A Dav Api.
         * @type      {Object}
         */
        dpm: function (res) {

            /* Stores the last request object.
             */
            var lastRequest = {};

            /* Stores the last requested resourceUrl.
             *
             * @see     #prepare
             */
            var resourceUrl;

            /* Stores the last Dom Node.
             */
            var resource;

            /* Stores last node searched for.
             *
             * @see     #getNodesByTag
             * @see     #seekToNode
             * @see     #eachNode
             */
            var lastNodeMatch = isArray (res) ? res : [res];

            if (isString (res)) {
                resourceUrl = res;
                // try { //Checking if the XML object is received as string/text
                    // resource = $.parseXML(res);
                // } catch (err) {
                    // //Wasn't an XML document definitely
                    // resource        = {};
                // }
                resource = {};
            }
            else {
                resource = res;
                resourceUrl = lastRequest.hasOwnProperty('url') ? lastRequest.url : '/';
            }

            var Api = function() {

                /***************************************
                 * Core Dav Methods
                 ***************************************/

                /* GET
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @url     http://www.webdav.org/specs/rfc2518.html#rfc.section.8.4
                 */
                this.get = function(cob) {
                    this.prepare(cob, 'GET');
                    return this.send(cob);
                };

                /* POST
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @url     http://www.webdav.org/specs/rfc2518.html#rfc.section.8.4
                 */
                this.post = function(cob) {
                    this.prepare(cob, 'POST');
                    return this.send(cob);
                };

                /* HEAD
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @url     http://www.webdav.org/specs/rfc2518.html#rfc.section.8.4
                 */
                this.head = function(cob) {
                    this.prepare(cob, 'HEAD');
                    return this.send(cob);
                };

                /* MKCOL
                 * Make a collection (folder).
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @href    http://greenbytes.de/tech/webdav/rfc4918.html#METHOD_MKCOL
                 * @see     #createFolder
                 */
                this.mkcol = function(cob) {
                    this.prepare(cob, 'MKCOL');
                    return this.send(cob);
                };

                /* PUT
                 * Puts a file (writes the content of the sent .body to a url)
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @href    http://greenbytes.de/tech/webdav/rfc4918.html#METHOD_PUT
                 * @see     #createFile
                 */
                this.put = function(cob) {
                    this.prepare(cob, 'PUT');
                    return this.send(cob);
                };

                /* DELETE
                 * Deletes a resource based on sent url.
                 * NOTE: using `remove` as `delete` is a protected JS word, and I don't
                 * want to uppercase method names just for this one.
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @href    http://greenbytes.de/tech/webdav/rfc4918.html#METHOD_DELETE
                 */
                this.remove = function(cob) {
                    this.prepare(cob, 'DELETE');
                    return this.send(cob);
                };

                /* REPORT
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @href    http://greenbytes.de/tech/webdav/rfc3253.html#METHOD_REPORT
                 * @href    http://greenbytes.de/tech/webdav/rfc3744.html#rfc.section.9.1
                 */
                this.report = function(cob) {
                    this.prepare(cob, 'REPORT');
                    return this.send(cob);
                };

                /* REPORT (*)
                 * @href http://greenbytes.de/tech/webdav/rfc3253.html#REPORT_version-tree
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @see     #REPORT
                 */
                this.getVersionTreeReport = function(cob) {
                    cob.data = XmlHeader + '\
                      <D:version-tree xmlns:D="DAV:">\
                        <D:prop>\
                          <D:version-name/>\
                          <D:creator-displayname/>\
                          <D:successor-set/>\
                          <D:predecessor-set/>\
                          <D:checkout-set/>\
                          <D:activity-set/>\
                          <D:getlastmodified/>\
                        </D:prop>\
                      </D:version-tree>';

                    extendBeforeSend(cob, function(xhr) {
                        xhr.setRequestHeader('Depth', 0);
                    });

                    this.prepare(cob, 'REPORT');
                    return this.send(cob);
                };

                /* CHECKOUT
                 * Checks out a VCR.
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @href    http://www.webdav.org/deltav/protocol/rfc3253.html#checkout-in-place.feature
                 */
                this.checkout = function(cob) {
                    this.prepare(cob, 'CHECKOUT');
                    return this.send(cob);
                };

                /* UNCHECKOUT
                 * Un-checks-out a VCR -- like a checkin, but without creating a new
                 * version, which is the same as rolling back to the pre-checkout state.
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @href    http://www.webdav.org/deltav/protocol/rfc3253.html#METHOD_UNCHECKOUT
                 */
                this.uncheckout = function(cob) {
                    this.prepare(cob, 'UNCHECKOUT');
                    return this.send(cob);
                };

                /* CHECKIN
                 * Checks in a VCR.
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @href     http://www.webdav.org/deltav/protocol/rfc3253.html#checkin.method.applied.to.a.version-controlled.resource
                 */
                this.checkin = function(cob) {
                    this.prepare(cob, 'CHECKIN');
                    return this.send(cob);
                };

                /* VERSION-CONTROL
                 * Creates a VCR (Version Controlled Resource)
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @href http://www.webdav.org/deltav/protocol/rfc3253.html#creating.a.version-controlled.resource
                 */
                this.versionControl = function(cob) {
                    this.prepare(cob, 'VERSION-CONTROL');
                    return this.send(cob);
                };

                /* PUT (*)
                 * An alias for PUT.  This allows users to create resources,
                 * optionally sending initial resource contents.
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @see     #put
                 */
                this.createFile = function(cob) {
                    return this.put(cob);
                };

                /* MKCOL (*)
                 * An alias for MKCOL.
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @see     #mkcol
                 */
                this.createFolder = function(cob) {
                    return this.mkcol(cob);
                };

                /* LOCK
                 * Locks a resource.
                 *
                 * @param   {Object}  cob     An object containing options for this call.
                 * @href    http://greenbytes.de/tech/webdav/rfc4918.html#METHOD_LOCK
                 */
                this.lock = function(cob) {
                    var user = cob.username || false;
                    var hdrs = cob.headers || {};

                    cob.lockscope = cob.lockscope || defaultLockScope;
                    cob.locktype = cob.locktype || defaultLockType;
                    cob.ifExists = !!cob.ifExists;

                    hdrs.Timeout = cob.lockTimeout || 'Second-' + defaultLockTimeout;
                    hdrs.Depth = cob.depth || defaultLockDepth;

                    /* If-Match * will stop a lock from being taken on an unmapped
                     * url -- which would normally create an locked, empty, resource.
                     */
                    if (cob.ifExists) {
                        hdrs['If-Match'] = '*';
                    }

                    /* Create the <owner> block.  If the user is authenticated,
                     * this becomes '/home/usernamehere'. NOTE: your Dav implementation
                     * may NOT use the /home/ protocol: it may not be a multiuser implementation,
                     * or uses a different directory, or something else.
                     */
                    if (user) {
                        cob.data = XmlHeader + '\
                            <D:lockinfo xmlns:D="DAV:">\
                                <D:lockscope>\
                                    <D:' + cob.lockscope + '/>\
                                </D:lockscope>\
                                <D:locktype>\
                                    <D:' + cob.locktype + '/>\
                                </D:locktype>\
                                <D:owner>' + userHomeDir + user + '</D:owner>\
                            </D:lockinfo>';

                        this.prepare(cob, 'LOCK');
                        return this.send(cob);
                    }
                };

                /* UNLOCK
                 * Attempts to unlock a resource
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @href    http://greenbytes.de/tech/webdav/rfc4918.html#METHOD_UNLOCK
                 */
                this.unlock = function(cob) {
                    var t = cob.lockToken;
                    /*
                     * Need a locktoken to unlock...
                     */
                    if (t) {
                        extendBeforeSend(cob, function(xhr) {
                            xhr.setRequestHeader('Lock-Token', '<' + t + '>');
                        });
                        this.prepare(cob, 'UNLOCK');
                        return this.send(cob);
                    }
                };

                /* PROPFIND
                 * You can always call this method directly, being sure to pass the
                 * proper Xml request body in call object (#data).  However, the
                 * intention is to have this be the 'engine' that powers useful
                 * variations on #propFind, doing the prepping for the user.
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @see     #getProperty
                 * @url     http://www.webdav.org/specs/rfc2518.html#METHOD_PROPFIND
                 */
                this.propFind = function(cob) {
                    this.prepare(cob, 'PROPFIND');

                    var depth = cob.headers.depth || 0;
                    /* Need to ensure that there is header information sent. Mainly,
                     * for a PROPFIND there must be a depth property. Note that the
                     * default depth is zero(0).
                     */
                    extendBeforeSend(cob, function (xhr) {
                        xhr.setRequestHeader('Depth', depth);
                        //xhr.setRequestHeader ('Authorization', 'Basic juanmvcalvellido:valencia');
                    });

                    return this.send(cob);
                };

                /* PROPPATCH
                 * Allows the modification of live properties on resources.  You
                 * may pass a property to set, or a property to remove, or both.
                 * Namespacing is also permitted.
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object w/ props:
                 *
                 * @example   #propPatch({
                 *              // standard jQuery#ajax properties
                 *              setProperty:    A property object, or an Array of
                 *                              property objects to set.
                 *              removeProperty: A property object, or an Array of
                 *                              property objects to remove.
                 *            });
                 *
                 * A property object: {
                 *  name:   'propertyName',
                 *  value:  'foo',
                 *  ns:     'ME:'
                 * };
                 *
                 * @see     #setProperty
                 * @url     http://www.webdav.org/specs/rfc2518.html#METHOD_PROPPATCH
                 */
                this.propPatch = function(cob) {
                    this.prepare(cob, 'PROPPATCH');

                    cob.setProperty = cob.setProperty ? isArray(cob.setProperty) ? cob.setProperty : [cob.setProperty] : [];
                    cob.removeProperty = cob.removeProperty ? isArray(cob.removeProperty) ? cob.removeProperty : [cob.removeProperty] : [];

                    cob.data = XmlHeader + '<D:propertyupdate xmlns:D="DAV:">';

                    var grouper = function(nm) {
                        var prop;
                        var props = cob[nm + 'Property'];
                        var block = '<D:' + nm + '><D:prop>';

                        for (var i = 0; i < props.length; i++) {
                            prop = props[i];
                            if (prop.name) {
                                block += '<P:' + prop.name + ' xmlns:P=\"' + (prop.ns || 'DAV:') + '\">' + prop.value + '</P:' + prop.name + '>';
                            }
                        }
                        block += '</D:prop></D:' + nm + '>';
                        return block;
                    };

                    cob.data += grouper('set');
                    cob.data += grouper('remove');

                    cob.data += '</D:propertyupdate>';

                    return this.send(cob);
                };

                /* PROPFIND (*)
                 * Fetches a property on a resource.
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @see     #propFind
                 */
                this.getProperty = function(cob) {
                    var props = cob.property || [];
                    var prop, i, xml;

                    for (i = 0; i < props.length; i++) {
                        prop = props[i];

                        /* Also want to allow the sending of a simple property string
                         * if only sending a property name. Instead of:
                         *   [{name: 'propname'}]
                         * send:
                         *   ['propname']
                         */
                        prop = isObject(prop) ? prop : {
                            name: prop
                        };
                        prop.ns = prop.ns || 'DAV:';

                        xml = '<P:' + prop.name + ' xmlns:P="' + prop.ns + '"/>';
                    }

                    cob.data = XmlHeader + '\
                        <propfind xmlns="DAV:">\
                            <prop>\
                                ' + xml + '\
                            </prop>\
                        </propfind>';

                    return this.propFind(cob);
                };

                /* PROPPATCH (*)
                 * Shortcut method for propPatch (set property).
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @see #propPatch
                 */
                this.setProperty = function(cob) {
                    if (cob.property) { //Only works if sent a property
                        cob.setProperty = cob.property;
                        delete cob.property;
                        return this.propPatch(cob);
                    }
                };

                /* PROPPATCH (*)
                 * Shortcut method for propPatch (remove property).
                 *
                 * @param   {Object}    cob   A jQuery#ajax call object.
                 * @see #propPatch
                 */
                this.removeProperty = function(cob) {
                    if (cob.property) { //Only works if sent a property
                        cob.removeProperty = cob.property;
                        delete cob.property;
                        return this.propPatch(cob);
                    }
                };


                /******************************************
                 * Node traversal/manipulation methods
                 ******************************************/

                /* Works exactly like getElementsByTagName, however introduces
                 * the ability to filter those results by namespace, which is
                 * important for handling DAV results.
                 *
                 * @param   {String}    tag     A tagName.
                 * @param   {String}    [ns]    A namespace, like 'DAV:'.
                 * @returns                     Array of matching nodes.
                 * @type    {Array}
                 */
                this.getNodesByTag = function(tag, ns) {

                    ns = ns || 'DAV:';

                    if (typeof resource.getElementsByTagNameNS === 'function') {
                        lastNodeMatch = resource.getElementsByTagNameNS(ns, tag);
                    } else {
                        /* getElementsByTagNameNS is supported on IE 9 onwards
                         * For IE mostly.
                         * setProperty is broken in IE < 8
                         */
                        resource.setProperty('SelectionLanguage', 'XPath');
                        resource.setProperty('SelectionNamespaces', 'xmlns:pref="' + ns + '"');

                        // //Solution #1 setAttribute (merged for IE >= 8)
                        // resource.setAttribute("SelectionLanguage", "XPath");
                        // resource.setAttribute("SelectionNamespaces", "xmlns:pref='" + ns + "'");

                        // //Solution #2 jQuery prop()
                        // $(resource).prop({
                            // 'SelectionLanguage':'XPath',
                            // 'SelectionNamespaces': 'xmlns:pref="' + ns + '"'
                        // });

                        lastNodeMatch = resource.selectNodes('//pref:' + tag);

                    }
                    return lastNodeMatch;
                };

                /* Sets `resource` to first node matched via #getNodesByTag, and
                 * returns `this`, allowing further processing.
                 *
                 * @param   {String}    tag     A tagName.
                 * @param   {String}    [ns]    A namespace, like 'DAV:'.
                 * @returns                     `this` object
                 * @type    {Object}
                 */
                this.seekToNode = function(tag, ns) {
                    var n = this.getNodesByTag(tag, ns);
                    resource = n[0] || resource;
                    return this;
                };

                /* Executes a function on each element of #lastNodeMatch
                 */
                this.eachNode = function(f) {
                    var i;

                    for (i = 0; i < lastNodeMatch.length; i++) {
                        f(lastNodeMatch[i], i);
                    }

                    return this;
                };

                /* Get the text contained by a node:
                 * <tagname>foo</tagname> = 'foo'.
                 *
                 * @type      {String}
                 */
                this.nodeText = function() {
                    var n, t = '';
                    for (var v = 0; v < resource.childNodes.length; v++) {
                        n = resource.childNodes[v];
                        t += n.textContent ? n.textContent : n.text;
                    }
                    return t.replace(/[\n\r\t]/g, '');
                };

                /* Returns the name of a node (the tag name, essentially). NOTE: result
                 * returned by #getNodesByTag is an Array.  As such, even if the
                 * node you have sought (by tagName) is unique, the value of `resource`
                 * is an Array. Note as well that if you do not send an index argument, the
                 * assumption made is that you want the [0] nodeName.  If you choose to use
                 * an index, no bounds checking is done, and you will get an error if outside.
                 * If `resource` is actually a single node, index is ignored.
                 *
                 * @param   {Number}    [i]     The index of
                 * @see #getNodesByTag
                 * @see #seekToNode
                 */
                this.nodeName = function(i) {
                    if (lastNodeMatch.length > 0) {
                        return lastNodeMatch[i || 0].nodeName.replace('D:', '');
                    }
                };

                /* Check if a <response> node is a collection.
                 * Basically it checks if there is a collection XML element inside
                 * a resourcetype property.
                 *
                 * @type    {Bool}
                 *
                 * @url     http://www.webdav.org/specs/rfc4918.html#PROPERTY_resourcetype
                 */
                this.isCollection = function() {
                    var is_collection = false;
                    if (lastNodeMatch.length > 0) {
                        this.seekToNode('resourcetype').eachNode(function(node) {
                            if (node.childNodes.length !== 0) {
                                var n = '';
                                for (var v = 0; v < node.childNodes.length; v++) {
                                    n = resource.childNodes[v];
                                    if (n.nodeName.replace('D:', '') === 'collection');
                                    is_collection = true;
                                }
                            }
                        });
                    }
                    return is_collection;
                };

                /* Check if a <response> node is a collection in a DPM server.
                 * It just checks for the node text of the iscollection tag
                 *
                 * @type    {Bool}
                 */
                this.isCollectionDPM = function() {
                    var is_collection = false;
                    if (lastNodeMatch.length > 0) {
                        this.seekToNode('iscollection');
                        is_collection = (resource.textContent == 1);
                    }
                    return is_collection;
                };


                /* Returns all the WebDAV/DPM defined properties of a
                 * <response> node
                 *
                 * @type    {JSON}
                 */
                this.getProperties = function() {
                    if (lastNodeMatch.length > 0) {
                        this.seekToNode('prop').eachNode(function(node) {
                            // if (node.childNodes.length !== 0) {
                                // console.log(node.childNodes[3].textContent);
                                // console.log(node.childNodes);
                            // }
                            var x = node.childNodes;
                            for (var i=0; i<x.length; i++){
                                console.log(x[i].nodeName);
                                // console.log(x[i][0].textContent);
                            }
                            // console.log(node);
                        });
                    }
                };

                /**************************************
                 * Resource Http query methods
                 **************************************/

                /* Prepares the DAV call.  Here we want to ensure integrity of
                 * call object, verify DAV method requested, set any authorization
                 * information (if necesssary), and return the modified call object.
                 * NOTE: This is called by the new jQuery extension #Dav; see below.
                 *
                 * @param   {Object}    cob       The call object.  This is the standard jQuery
                 *                                #ajax object. Note the defaults set here.
                 * @param   {String}    typ       The call type.
                 */
                this.prepare = function (cob, typ) {

                    cob = cob || {};
                    cob.url = resourceUrl;
                    cob.headers = cob.headers || {};

                    //Ensure that we have a method set, defaulting to GET.
                    cob.type = typ || 'GET';

                    /* WebDAV servers usually respond in XML.  Some Dav methods will
                     * not return anything at all, or return an empty response.  This
                     * matters to jQuery in that the #success method of a jQuery.ajax
                     * options object won't fire if the reponse could not be parsed
                     * (although the #complete method will).  You should be aware of what
                     * the #dataType of your ajax response will be, and set it appropriately.
                     */
                    cob.dataType = cob.dataType || 'xml';

                    /* For Http methods not supported by the current browser, we use a
                     * specialized PUT with query.  NOTE that this particular behaviour exists
                     * on the LimeBits server only.  If you do not extend the same support to
                     * your Dav server, it is simple to be optimistic and rewrite so that the
                     * browser attempts to send Dav Http methods, and will simply fail if it
                     * cannot.  See jQuery documentation on #ajax options regarding non-regular
                     * Http method failures, for option #type.
                     */
                    if (!methodSupported(cob.type)) {

                        /* Fetch authentication data, which is necessary for the execution of
                         * any methods, by this user, which are DENY-d to 'unauthenticated' principal.
                         */
                        var auth = document.cookie.match(new RegExp('(^|;)\\s*' + escape('auth') + '=([^;\\s]*)'));
                        auth = auth ? unescape(auth[2]) : null;

                        if (auth) {
                            auth = '&auth=' + auth;
                        } else {
                            auth = '';
                        }

                        cob.url += '?webdav-method=' + cob.type.toUpperCase() + auth;
                        cob.type = 'POST';
                    }
                };

                /* Here the actual HTTP petition is sent via ajax.
                 *
                 * @param   {Object}    cob     jQuery call object.
                 */
                this.send = function(cob) {
                    lastRequest = $.ajax(cob);
                    return lastRequest;
                };

                /************************************************************
                 * Some optional, additional methods you might want to use.
                 ************************************************************/

                this.getAllProperties = function(cob) {
                    cob.data = XmlHeader + '\
                      <D:propfind xmlns:D="DAV:">\
                        <D:allprop/>';

                    if (isArray(cob.includes)) {
                        cob.data += '<D:include>';

                        for (var i = 0; i < d.includes.length; i++) {
                            cob.body += '<D:' + d.includes[i] + '/>';
                        }

                        cob.data += '</D:include>';
                    }

                    cob.data += '</D:propfind>';
                    return this.propFind(cob);
                };

                /* Simply a depth (1) #getAllProperties -- an alias that should
                 * make it easier to follow what is going on, dpm.readFolder()
                 * instead of dpm.getAllProperties({ depth: 1 });
                 */
                this.readFolder = function(cob) {
                    /* Since this ends up in a #propFind, we only need to
                     * indicate the headers -- the extension of the headers
                     * will be done in the #propFind method.  We don't need
                     * to run #extendBeforeSend manually, in other words.
                     */
                    cob.headers = {
                        depth: 1
                    };

                    return this.getAllProperties(cob);
                };

            }; //End of Api definition

            return new Api();

        } //End of dpm extension definition

    }); //End of jQuery.fn.extend

})(jQuery);
